﻿//-----------------------------------------------------------------------
// FILENAME: LibraryContext.cs
// PROJECT: DbContextLib
// SOLUTION: FitnessApp
// DATE CREATED: 22/02/2024
// AUTHOR: Antoine PEREDERII
//-----------------------------------------------------------------------

using Entities;
using Microsoft.EntityFrameworkCore;

namespace DbContextLib
{
    /// <summary>
    /// Represents the database context for the FitnessApp.
    /// </summary>
    public class LibraryContext : DbContext
    {
        /// <summary>
        /// Gets or sets the set of athletes.
        /// </summary>
        public DbSet<AthleteEntity> AthletesSet { get; set; }

        /// <summary>
        /// Gets or sets the set of activities.
        /// </summary>
        public DbSet<ActivityEntity> ActivitiesSet { get; set; }

        /// <summary>
        /// Gets or sets the set of data sources.
        /// </summary>
        public DbSet<DataSourceEntity> DataSourcesSet { get; set; }

        /// <summary>
        /// Gets or sets the set of heart rates.
        /// </summary>
        public DbSet<HeartRateEntity> HeartRatesSet { get; set; }

        /// <summary>
        /// Gets or sets the set of notifications.
        /// </summary>
        public DbSet<NotificationEntity> NotificationsSet { get; set; }

        /// <summary>
        /// Gets or sets the set of statistics.
        /// </summary>
        public DbSet<StatisticEntity> StatisticsSet { get; set; }

        /// <summary>
        /// Gets or sets the set of trainings.
        /// </summary>
        public DbSet<TrainingEntity> TrainingsSet { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="LibraryContext"/> class.
        /// </summary>
        public LibraryContext() : base() { }

        /// <summary>
        /// Initializes a new instance of the <see cref="LibraryContext"/> class with the specified options.
        /// </summary>
        /// <param name="options">The options for the context.</param>
        public LibraryContext(DbContextOptions<LibraryContext> options) : base(options) { }

        /// <summary>
        /// Configures the database options if they are not already configured.
        /// </summary>
        /// <param name="optionsBuilder">The options builder instance.</param>
        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            if (!optionsBuilder.IsConfigured)
            {
                optionsBuilder.UseSqlite($"Data Source=uca.HeartTrack.db");
            }
        }

        /// <summary>
        /// Configures the model for the library context.
        /// </summary>
        /// <param name="modelBuilder">The model builder instance.</param>
        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            base.OnModelCreating(modelBuilder);

            modelBuilder.Entity<ActivityEntity>()
                .HasKey(a => a.IdActivity);
            //generation mode (at insertion)
            modelBuilder.Entity<ActivityEntity>()
                        .Property(a => a.IdActivity)
                        .ValueGeneratedOnAdd();

            //primary key of HeartRateEntity
            modelBuilder.Entity<HeartRateEntity>()
                        .HasKey(h => h.IdHeartRate);
            //generation mode (at insertion)
            modelBuilder.Entity<HeartRateEntity>()
                        .Property(h => h.IdHeartRate)
                        .ValueGeneratedOnAdd();

            //primary key of DataSourceEntity
            modelBuilder.Entity<DataSourceEntity>()
                        .HasKey(d => d.IdSource);
            //generation mode (at insertion)
            modelBuilder.Entity<DataSourceEntity>()
                        .Property(d => d.IdSource)
                        .ValueGeneratedOnAdd();

            //primary key of AthleteEntity
            modelBuilder.Entity<AthleteEntity>()
                        .HasKey(at => at.IdAthlete);
            //generation mode (at insertion)
            modelBuilder.Entity<AthleteEntity>()
                        .Property(at => at.IdAthlete)
                        .ValueGeneratedOnAdd();

            //primary key of StatisticEntity
            modelBuilder.Entity<StatisticEntity>()
                        .HasKey(s => s.IdStatistic);
            //generation mode (at insertion)
            modelBuilder.Entity<StatisticEntity>()
                        .Property(s => s.IdStatistic)
                        .ValueGeneratedOnAdd();

            //primary key of 
            modelBuilder.Entity<TrainingEntity>()
                        .HasKey(t => t.IdTraining);
            //generation mode (at insertion)
            modelBuilder.Entity<TrainingEntity>()
                        .Property(t => t.IdTraining)
                        .ValueGeneratedOnAdd();
            
            //primary key of 
            modelBuilder.Entity<NotificationEntity>()
                        .HasKey(n => n.IdNotif);
            //generation mode (at insertion)
            modelBuilder.Entity<NotificationEntity>()
                        .Property(n => n.IdNotif)
                        .ValueGeneratedOnAdd();

            modelBuilder.Entity<AthleteEntity>()
                .HasMany(at => at.Trainings)
                .WithOne(n => n.Athlete)
                .HasForeignKey(n => n.AthleteId)
                .IsRequired();

            modelBuilder.Entity<AthleteEntity>()
                .HasMany(at => at.Trainings)
                .WithOne(t => t.Athlete)
                .HasForeignKey(t => t.AthleteId)
                .IsRequired();

            modelBuilder.Entity<AthleteEntity>()
                .HasMany(at => at.Statistics)
                .WithOne(s => s.Athlete)
                .HasForeignKey(s => s.AthleteId)
                .IsRequired();

            modelBuilder.Entity<AthleteEntity>()
                .HasMany(at => at.Activities)
                .WithOne(a => a.Athlete)
                .HasForeignKey(a => a.AthleteId)
                .IsRequired();

            modelBuilder.Entity<DataSourceEntity>()
                .HasMany(d => d.Activities)
                .WithOne(a => a.DataSource)
                .HasForeignKey(a => a.DataSourceId)
                .IsRequired();

            modelBuilder.Entity<ActivityEntity>()
                .HasMany(a => a.HeartRates)
                .WithOne(h => h.Activity)
                .HasForeignKey(h => h.ActivityId)
                .IsRequired();
            
            modelBuilder.Entity<DataSourceEntity>()
                .HasMany(ds => ds.Activities)
                .WithOne(at => at.DataSource)
                .HasForeignKey(at => at.DataSourceId)
                .IsRequired(false);
        }
    }
}